
from llava.model.builder import load_pretrained_model
from llava.mm_utils import get_model_name_from_path, process_images, tokenizer_image_token
from llava.constants import IMAGE_TOKEN_INDEX, DEFAULT_IMAGE_TOKEN, DEFAULT_IM_START_TOKEN, DEFAULT_IM_END_TOKEN, IGNORE_INDEX
from llava.conversation import conv_templates, SeparatorStyle

from PIL import Image
import requests
import copy
import torch

import sys
import warnings

warnings.filterwarnings("ignore")
pretrained = "/data/home/zbz5349/WorkSpace/Model/llava-critic-7b"
model_name = "llava_qwen"
device = "cuda"
device_map = "auto"
tokenizer, model, image_processor, max_length = load_pretrained_model(pretrained, None, model_name, device_map=device_map)  # Add any other thing you want to pass in llava_model_args
model.eval()


 

def infer(image_path, content):

    image = Image.open(image_path)
    image_tensor = process_images([image], image_processor, model.config)
    image_tensor = [_image.to(dtype=torch.float16, device=device) for _image in image_tensor]

    conv_template = "qwen_1_5"  # Make sure you use correct chat template for different models
    # question = DEFAULT_IMAGE_TOKEN + "\nWhat is shown in this image?"
    # question = DEFAULT_IMAGE_TOKEN + "\n " + content #+ "What is shown in this image?"

    final_prompt = 'From 0 to 100, how much do you rate for this Text Caption in terms of the correct and comprehensive description of the image? Do not dominant the rating by a single attribute such as recognition correctness, but a overall rating on the object/scene appearance, position, pose, action, shape, etc., and contents in the background. Do not consider the appropriateness or sensitive descriptors, such as “middle-aged western man”, judge based on if it has correct specifications of the object and scenes in image. Provide a few lines for explanation and the rate number at last after  “Final Score:”.'
    question = DEFAULT_IMAGE_TOKEN + "\n " + content  + final_prompt


    conv = copy.deepcopy(conv_templates[conv_template])
    conv.append_message(conv.roles[0], question)
    conv.append_message(conv.roles[1], None)
    prompt_question = conv.get_prompt()

    input_ids = tokenizer_image_token(prompt_question, tokenizer, IMAGE_TOKEN_INDEX, return_tensors="pt").unsqueeze(0).to(device)
    image_sizes = [image.size]


    cont = model.generate(
        input_ids,
        images=image_tensor,
        image_sizes=image_sizes,
        do_sample=False,
        temperature=0,
        max_new_tokens=4096,
    )
    text_outputs = tokenizer.batch_decode(cont, skip_special_tokens=True)
    print(text_outputs)
    return text_outputs




import json

def read_json(file_path): 
    with open(file_path, 'r', encoding='utf-8') as file:
        data = json.load(file)
    return data

def write_json(file_path, data):
    with open(file_path, 'w', encoding='utf-8') as file:
        json.dump(data, file, ensure_ascii=False, indent=4)



data = read_json('/data/home/zbz5349/WorkSpace/DataSet/llava_image_tune_.json')


index1 = int(len(data)/3)
index2 = index1*2

all_data = []
from tqdm import tqdm

counter = 0  # 初始化计数器
for i in tqdm(data[index2:]):
    content = " ".join([conv["value"] for conv in i["conversations"]])
    image_path = '/data/home/zbz5349/WorkSpace/DataSet/' + i['image']
    result = infer(image_path, content)
    
    temp = {}
    temp['question'] = content
    temp['answer'] = result
    all_data.append(temp)
    
    counter += 1  # 增加计数器
    
    # 每200次保存一次
    if counter % 200 == 0:
        # 保存文件，文件名中可以加入计数器标记
        write_json(f'/data/home/zbz5349/WorkSpace/LLaVA-NeXT/data/infer_demo_score_600k_{counter}.json', all_data)

# 循环结束后再保存一次，以确保剩余数据也保存
write_json('/data/home/zbz5349/WorkSpace/LLaVA-NeXT/data/infer_demo_score_600k_final.json', all_data)



